CREATE TABLE [dbo].[authors] (
    [au_id]    [dbo].[id]   NOT NULL,
    [au_lname] VARCHAR (40) NOT NULL,
    [au_fname] VARCHAR (20) NOT NULL,
    [phone]    CHAR (12)    DEFAULT ('UNKNOWN') NOT NULL,
    [address]  VARCHAR (40) NULL,
    [city]     VARCHAR (20) NULL,
    [state]    CHAR (2)     NULL,
    [zip]      CHAR (5)     NULL,
    [contract] BIT          NOT NULL,
    CHECK ([au_id] LIKE '[0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9][0-9][0-9]'),
    CHECK ([zip] LIKE '[0-9][0-9][0-9][0-9][0-9]')
);

GO
CREATE TABLE [dbo].[discounts] (
    [discounttype] VARCHAR (40)   NOT NULL,
    [stor_id]      CHAR (4)       NULL,
    [lowqty]       SMALLINT       NULL,
    [highqty]      SMALLINT       NULL,
    [discount]     DECIMAL (4, 2) NOT NULL,
    FOREIGN KEY ([stor_id]) REFERENCES [dbo].[stores] ([stor_id])
);

GO
CREATE TABLE [dbo].[employee] (
    [emp_id]    [dbo].[empid] NOT NULL,
    [fname]     VARCHAR (20)  NOT NULL,
    [minit]     CHAR (1)      NULL,
    [lname]     VARCHAR (30)  NOT NULL,
    [job_id]    SMALLINT      DEFAULT ((1)) NOT NULL,
    [job_lvl]   TINYINT       DEFAULT ((10)) NULL,
    [pub_id]    CHAR (4)      DEFAULT ('9952') NOT NULL,
    [hire_date] DATETIME      DEFAULT (getdate()) NOT NULL,
    FOREIGN KEY ([job_id]) REFERENCES [dbo].[jobs] ([job_id]),
    FOREIGN KEY ([pub_id]) REFERENCES [dbo].[publishers] ([pub_id])
);

GO
CREATE TABLE [dbo].[jobs] (
    [job_id]   SMALLINT     IDENTITY (1, 1) NOT NULL,
    [job_desc] VARCHAR (50) DEFAULT ('New Position - title not formalized yet') NOT NULL,
    [min_lvl]  TINYINT      NOT NULL,
    [max_lvl]  TINYINT      NOT NULL,
    PRIMARY KEY CLUSTERED ([job_id] ASC),
    CHECK ([max_lvl] <= (250)),
    CHECK ([min_lvl] >= (10))
);

GO
CREATE TABLE [dbo].[pub_info] (
    [pub_id]  CHAR (4) NOT NULL,
    [logo]    IMAGE    NULL,
    [pr_info] TEXT     NULL,
    FOREIGN KEY ([pub_id]) REFERENCES [dbo].[publishers] ([pub_id])
);

GO
CREATE TABLE [dbo].[publishers] (
    [pub_id]   CHAR (4)     NOT NULL,
    [pub_name] VARCHAR (40) NULL,
    [city]     VARCHAR (20) NULL,
    [state]    CHAR (2)     NULL,
    [country]  VARCHAR (30) DEFAULT ('USA') NULL,
    CHECK ([pub_id] = '1756'
           OR [pub_id] = '1622'
           OR [pub_id] = '0877'
           OR [pub_id] = '0736'
           OR [pub_id] = '1389'
           OR [pub_id] LIKE '99[0-9][0-9]')
);

GO
CREATE TABLE [dbo].[roysched] (
    [title_id] [dbo].[tid] NOT NULL,
    [lorange]  INT         NULL,
    [hirange]  INT         NULL,
    [royalty]  INT         NULL,
    FOREIGN KEY ([title_id]) REFERENCES [dbo].[titles] ([title_id])
);

GO
CREATE TABLE [dbo].[sales] (
    [stor_id]  CHAR (4)     NOT NULL,
    [ord_num]  VARCHAR (20) NOT NULL,
    [ord_date] DATETIME     NOT NULL,
    [qty]      SMALLINT     NOT NULL,
    [payterms] VARCHAR (12) NOT NULL,
    [title_id] [dbo].[tid]  NOT NULL,
    FOREIGN KEY ([stor_id]) REFERENCES [dbo].[stores] ([stor_id]),
    FOREIGN KEY ([title_id]) REFERENCES [dbo].[titles] ([title_id])
);

GO
CREATE TABLE [dbo].[stores] (
    [stor_id]      CHAR (4)     NOT NULL,
    [stor_name]    VARCHAR (40) NULL,
    [stor_address] VARCHAR (40) NULL,
    [city]         VARCHAR (20) NULL,
    [state]        CHAR (2)     NULL,
    [zip]          CHAR (5)     NULL
);

GO
CREATE TABLE [dbo].[tbl_Name] (
    [id]         INT            IDENTITY (1, 1) NOT NULL,
    [ColumnName] NVARCHAR (100) NULL,
    PRIMARY KEY CLUSTERED ([id] ASC)
);

GO
CREATE TABLE [dbo].[titleauthor] (
    [au_id]      [dbo].[id]  NOT NULL,
    [title_id]   [dbo].[tid] NOT NULL,
    [au_ord]     TINYINT     NULL,
    [royaltyper] INT         NULL,
    FOREIGN KEY ([au_id]) REFERENCES [dbo].[authors] ([au_id]),
    FOREIGN KEY ([title_id]) REFERENCES [dbo].[titles] ([title_id])
);

GO
CREATE TABLE [dbo].[titles] (
    [title_id]  [dbo].[tid]   NOT NULL,
    [title]     VARCHAR (80)  NOT NULL,
    [type]      CHAR (12)     DEFAULT ('UNDECIDED') NOT NULL,
    [pub_id]    CHAR (4)      NULL,
    [price]     MONEY         NULL,
    [advance]   MONEY         NULL,
    [royalty]   INT           NULL,
    [ytd_sales] INT           NULL,
    [notes]     VARCHAR (200) NULL,
    [pubdate]   DATETIME      DEFAULT (getdate()) NOT NULL,
    FOREIGN KEY ([pub_id]) REFERENCES [dbo].[publishers] ([pub_id])
);

GO
CREATE VIEW [dbo].[titleview]
AS
SELECT title,
       au_ord,
       au_lname,
       price,
       ytd_sales,
       pub_id
FROM   authors, titles, titleauthor
WHERE  authors.au_id = titleauthor.au_id
       AND titles.title_id = titleauthor.title_id;

GO
ALTER TABLE [dbo].[titles]
    ADD FOREIGN KEY ([pub_id]) REFERENCES [dbo].[publishers] ([pub_id]);

GO
ALTER TABLE [dbo].[titleauthor]
    ADD FOREIGN KEY ([au_id]) REFERENCES [dbo].[authors] ([au_id]);

GO
ALTER TABLE [dbo].[titleauthor]
    ADD FOREIGN KEY ([title_id]) REFERENCES [dbo].[titles] ([title_id]);

GO
ALTER TABLE [dbo].[sales]
    ADD FOREIGN KEY ([stor_id]) REFERENCES [dbo].[stores] ([stor_id]);

GO
ALTER TABLE [dbo].[sales]
    ADD FOREIGN KEY ([title_id]) REFERENCES [dbo].[titles] ([title_id]);

GO
ALTER TABLE [dbo].[roysched]
    ADD FOREIGN KEY ([title_id]) REFERENCES [dbo].[titles] ([title_id]);

GO
ALTER TABLE [dbo].[discounts]
    ADD FOREIGN KEY ([stor_id]) REFERENCES [dbo].[stores] ([stor_id]);

GO
ALTER TABLE [dbo].[pub_info]
    ADD FOREIGN KEY ([pub_id]) REFERENCES [dbo].[publishers] ([pub_id]);

GO
ALTER TABLE [dbo].[employee]
    ADD FOREIGN KEY ([job_id]) REFERENCES [dbo].[jobs] ([job_id]);

GO
ALTER TABLE [dbo].[employee]
    ADD FOREIGN KEY ([pub_id]) REFERENCES [dbo].[publishers] ([pub_id]);

GO
ALTER TABLE [dbo].[authors]
    ADD DEFAULT ('UNKNOWN') FOR [phone];

GO
ALTER TABLE [dbo].[publishers]
    ADD DEFAULT ('USA') FOR [country];

GO
ALTER TABLE [dbo].[titles]
    ADD DEFAULT ('UNDECIDED') FOR [type];

GO
ALTER TABLE [dbo].[titles]
    ADD DEFAULT (getdate()) FOR [pubdate];

GO
ALTER TABLE [dbo].[jobs]
    ADD DEFAULT ('New Position - title not formalized yet') FOR [job_desc];

GO
ALTER TABLE [dbo].[employee]
    ADD DEFAULT ((1)) FOR [job_id];

GO
ALTER TABLE [dbo].[employee]
    ADD DEFAULT ((10)) FOR [job_lvl];

GO
ALTER TABLE [dbo].[employee]
    ADD DEFAULT ('9952') FOR [pub_id];

GO
ALTER TABLE [dbo].[employee]
    ADD DEFAULT (getdate()) FOR [hire_date];

GO
ALTER TABLE [dbo].[authors]
    ADD CHECK ([au_id] LIKE '[0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9][0-9][0-9]');

GO
ALTER TABLE [dbo].[authors]
    ADD CHECK ([zip] LIKE '[0-9][0-9][0-9][0-9][0-9]');

GO
ALTER TABLE [dbo].[publishers]
    ADD CHECK ([pub_id] = '1756'
               OR [pub_id] = '1622'
               OR [pub_id] = '0877'
               OR [pub_id] = '0736'
               OR [pub_id] = '1389'
               OR [pub_id] LIKE '99[0-9][0-9]');

GO
ALTER TABLE [dbo].[jobs]
    ADD CHECK ([min_lvl] >= (10));

GO
ALTER TABLE [dbo].[jobs]
    ADD CHECK ([max_lvl] <= (250));

GO
ALTER TABLE [dbo].[employee]
    ADD CONSTRAINT [CK_emp_id] CHECK ([emp_id] LIKE '[A-Z][A-Z][A-Z][1-9][0-9][0-9][0-9][0-9][FM]'
                                      OR [emp_id] LIKE '[A-Z]-[A-Z][1-9][0-9][0-9][0-9][0-9][FM]');

GO
ALTER TABLE [dbo].[jobs]
    ADD PRIMARY KEY CLUSTERED ([job_id] ASC);

GO
ALTER TABLE [dbo].[tbl_Name]
    ADD PRIMARY KEY CLUSTERED ([id] ASC);

GO
ALTER TABLE [dbo].[employee]
    ADD CONSTRAINT [PK_emp_id] PRIMARY KEY NONCLUSTERED ([emp_id] ASC);

GO
ALTER TABLE [dbo].[stores]
    ADD CONSTRAINT [UPK_storeid] PRIMARY KEY CLUSTERED ([stor_id] ASC);

GO
ALTER TABLE [dbo].[authors]
    ADD CONSTRAINT [UPKCL_auidind] PRIMARY KEY CLUSTERED ([au_id] ASC);

GO
ALTER TABLE [dbo].[publishers]
    ADD CONSTRAINT [UPKCL_pubind] PRIMARY KEY CLUSTERED ([pub_id] ASC);

GO
ALTER TABLE [dbo].[pub_info]
    ADD CONSTRAINT [UPKCL_pubinfo] PRIMARY KEY CLUSTERED ([pub_id] ASC);

GO
ALTER TABLE [dbo].[sales]
    ADD CONSTRAINT [UPKCL_sales] PRIMARY KEY CLUSTERED ([stor_id] ASC, [ord_num] ASC, [title_id] ASC);

GO
ALTER TABLE [dbo].[titleauthor]
    ADD CONSTRAINT [UPKCL_taind] PRIMARY KEY CLUSTERED ([au_id] ASC, [title_id] ASC);

GO
ALTER TABLE [dbo].[titles]
    ADD CONSTRAINT [UPKCL_titleidind] PRIMARY KEY CLUSTERED ([title_id] ASC);

GO
CREATE NONCLUSTERED INDEX [aunmind]
    ON [dbo].[authors]([au_lname] ASC, [au_fname] ASC);

GO
CREATE CLUSTERED INDEX [employee_ind]
    ON [dbo].[employee]([lname] ASC, [fname] ASC, [minit] ASC);

GO
CREATE NONCLUSTERED INDEX [titleidind]
    ON [dbo].[roysched]([title_id] ASC);

GO
CREATE NONCLUSTERED INDEX [titleidind]
    ON [dbo].[sales]([title_id] ASC);

GO
CREATE NONCLUSTERED INDEX [auidind]
    ON [dbo].[titleauthor]([au_id] ASC);

GO
CREATE NONCLUSTERED INDEX [titleidind]
    ON [dbo].[titleauthor]([title_id] ASC);

GO
CREATE NONCLUSTERED INDEX [titleind]
    ON [dbo].[titles]([title] ASC);

GO
CREATE PROCEDURE [dbo].[byroyalty]
@percentage INT
AS
SELECT au_id
FROM   titleauthor
WHERE  titleauthor.royaltyper = @percentage;

GO
CREATE TRIGGER [dbo].[employee_insupd]
    ON [dbo].[employee]
    FOR INSERT, UPDATE
    AS DECLARE @min_lvl AS TINYINT, @max_lvl AS TINYINT, @emp_lvl AS TINYINT, @job_id AS SMALLINT;
       SELECT @min_lvl = min_lvl,
              @max_lvl = max_lvl,
              @emp_lvl = i.job_lvl,
              @job_id = i.job_id
       FROM   employee AS e, jobs AS j, inserted AS i
       WHERE  e.emp_id = i.emp_id
              AND i.job_id = j.job_id;
       IF (@job_id = 1)
          AND (@emp_lvl <> 10)
           BEGIN
               RAISERROR ('Job id 1 expects the default level of 10.', 16, 1);
               ROLLBACK;
           END
       ELSE
           IF NOT (@emp_lvl BETWEEN @min_lvl AND @max_lvl)
               BEGIN
                   RAISERROR ('The level for job_id:%d should be between %d and %d.', 16, 1, @job_id, @min_lvl, @max_lvl);
                   ROLLBACK;
               END

GO
CREATE TYPE [dbo].[empid]
    FROM CHAR (9) NOT NULL;

GO
CREATE TYPE [dbo].[id]
    FROM VARCHAR (11) NOT NULL;

GO
CREATE TYPE [dbo].[tid]
    FROM VARCHAR (6) NOT NULL;

GO
GRANT CREATE DEFAULT TO [guest]
    AS [dbo];

GO
GRANT CREATE PROCEDURE TO [guest]
    AS [dbo];

GO
GRANT CREATE PROCEDURE TO PUBLIC
    AS [dbo];

GO
GRANT CREATE RULE TO [guest]
    AS [dbo];

GO
GRANT CREATE TABLE TO [guest]
    AS [dbo];

GO
GRANT CREATE VIEW TO [guest]
    AS [dbo];

GO
GRANT DELETE
    ON OBJECT::[dbo].[authors] TO [guest]
    AS [dbo];

GO
GRANT DELETE
    ON OBJECT::[dbo].[discounts] TO [guest]
    AS [dbo];

GO
GRANT DELETE
    ON OBJECT::[dbo].[employee] TO [guest]
    AS [dbo];

GO
GRANT DELETE
    ON OBJECT::[dbo].[jobs] TO [guest]
    AS [dbo];

GO
GRANT DELETE
    ON OBJECT::[dbo].[pub_info] TO [guest]
    AS [dbo];

GO
GRANT DELETE
    ON OBJECT::[dbo].[publishers] TO [guest]
    AS [dbo];

GO
GRANT DELETE
    ON OBJECT::[dbo].[roysched] TO [guest]
    AS [dbo];

GO
GRANT DELETE
    ON OBJECT::[dbo].[sales] TO [guest]
    AS [dbo];

GO
GRANT DELETE
    ON OBJECT::[dbo].[stores] TO [guest]
    AS [dbo];

GO
GRANT DELETE
    ON OBJECT::[dbo].[titleauthor] TO [guest]
    AS [dbo];

GO
GRANT DELETE
    ON OBJECT::[dbo].[titles] TO [guest]
    AS [dbo];

GO
GRANT DELETE
    ON OBJECT::[dbo].[titleview] TO [guest]
    AS [dbo];

GO
GRANT EXECUTE
    ON OBJECT::[dbo].[byroyalty] TO [guest]
    AS [dbo];

GO
GRANT EXECUTE
    ON OBJECT::[dbo].[byroyalty] TO PUBLIC
    AS [dbo];

GO
GRANT INSERT
    ON OBJECT::[dbo].[authors] TO [guest]
    AS [dbo];

GO
GRANT INSERT
    ON OBJECT::[dbo].[discounts] TO [guest]
    AS [dbo];

GO
GRANT INSERT
    ON OBJECT::[dbo].[employee] TO [guest]
    AS [dbo];

GO
GRANT INSERT
    ON OBJECT::[dbo].[jobs] TO [guest]
    AS [dbo];

GO
GRANT INSERT
    ON OBJECT::[dbo].[pub_info] TO [guest]
    AS [dbo];

GO
GRANT INSERT
    ON OBJECT::[dbo].[publishers] TO [guest]
    AS [dbo];

GO
GRANT INSERT
    ON OBJECT::[dbo].[roysched] TO [guest]
    AS [dbo];

GO
GRANT INSERT
    ON OBJECT::[dbo].[sales] TO [guest]
    AS [dbo];

GO
GRANT INSERT
    ON OBJECT::[dbo].[stores] TO [guest]
    AS [dbo];

GO
GRANT INSERT
    ON OBJECT::[dbo].[titleauthor] TO [guest]
    AS [dbo];

GO
GRANT INSERT
    ON OBJECT::[dbo].[titles] TO [guest]
    AS [dbo];

GO
GRANT INSERT
    ON OBJECT::[dbo].[titleview] TO [guest]
    AS [dbo];

GO
GRANT REFERENCES
    ON OBJECT::[dbo].[authors] TO [guest]
    AS [dbo];

GO
GRANT REFERENCES
    ON OBJECT::[dbo].[discounts] TO [guest]
    AS [dbo];

GO
GRANT REFERENCES
    ON OBJECT::[dbo].[employee] TO [guest]
    AS [dbo];

GO
GRANT REFERENCES
    ON OBJECT::[dbo].[jobs] TO [guest]
    AS [dbo];

GO
GRANT REFERENCES
    ON OBJECT::[dbo].[pub_info] TO [guest]
    AS [dbo];

GO
GRANT REFERENCES
    ON OBJECT::[dbo].[publishers] TO [guest]
    AS [dbo];

GO
GRANT REFERENCES
    ON OBJECT::[dbo].[roysched] TO [guest]
    AS [dbo];

GO
GRANT REFERENCES
    ON OBJECT::[dbo].[sales] TO [guest]
    AS [dbo];

GO
GRANT REFERENCES
    ON OBJECT::[dbo].[stores] TO [guest]
    AS [dbo];

GO
GRANT REFERENCES
    ON OBJECT::[dbo].[titleauthor] TO [guest]
    AS [dbo];

GO
GRANT REFERENCES
    ON OBJECT::[dbo].[titles] TO [guest]
    AS [dbo];

GO
GRANT REFERENCES
    ON OBJECT::[dbo].[titleview] TO [guest]
    AS [dbo];

GO
GRANT REFERENCES
    ON TYPE::[dbo].[empid] TO PUBLIC
    AS [dbo];

GO
GRANT REFERENCES
    ON TYPE::[dbo].[id] TO PUBLIC
    AS [dbo];

GO
GRANT REFERENCES
    ON TYPE::[dbo].[tid] TO PUBLIC
    AS [dbo];

GO
GRANT SELECT
    ON OBJECT::[dbo].[authors] TO [guest]
    AS [dbo];

GO
GRANT SELECT
    ON OBJECT::[dbo].[discounts] TO [guest]
    AS [dbo];

GO
GRANT SELECT
    ON OBJECT::[dbo].[employee] TO [guest]
    AS [dbo];

GO
GRANT SELECT
    ON OBJECT::[dbo].[jobs] TO [guest]
    AS [dbo];

GO
GRANT SELECT
    ON OBJECT::[dbo].[pub_info] TO [guest]
    AS [dbo];

GO
GRANT SELECT
    ON OBJECT::[dbo].[publishers] TO [guest]
    AS [dbo];

GO
GRANT SELECT
    ON OBJECT::[dbo].[roysched] TO [guest]
    AS [dbo];

GO
GRANT SELECT
    ON OBJECT::[dbo].[sales] TO [guest]
    AS [dbo];

GO
GRANT SELECT
    ON OBJECT::[dbo].[stores] TO [guest]
    AS [dbo];

GO
GRANT SELECT
    ON OBJECT::[dbo].[titleauthor] TO [guest]
    AS [dbo];

GO
GRANT SELECT
    ON OBJECT::[dbo].[titles] TO [guest]
    AS [dbo];

GO
GRANT SELECT
    ON OBJECT::[dbo].[titleview] TO [guest]
    AS [dbo];

GO
GRANT UPDATE
    ON OBJECT::[dbo].[authors] TO [guest]
    AS [dbo];

GO
GRANT UPDATE
    ON OBJECT::[dbo].[discounts] TO [guest]
    AS [dbo];

GO
GRANT UPDATE
    ON OBJECT::[dbo].[employee] TO [guest]
    AS [dbo];

GO
GRANT UPDATE
    ON OBJECT::[dbo].[jobs] TO [guest]
    AS [dbo];

GO
GRANT UPDATE
    ON OBJECT::[dbo].[pub_info] TO [guest]
    AS [dbo];

GO
GRANT UPDATE
    ON OBJECT::[dbo].[publishers] TO [guest]
    AS [dbo];

GO
GRANT UPDATE
    ON OBJECT::[dbo].[roysched] TO [guest]
    AS [dbo];

GO
GRANT UPDATE
    ON OBJECT::[dbo].[sales] TO [guest]
    AS [dbo];

GO
GRANT UPDATE
    ON OBJECT::[dbo].[stores] TO [guest]
    AS [dbo];

GO
GRANT UPDATE
    ON OBJECT::[dbo].[titleauthor] TO [guest]
    AS [dbo];

GO
GRANT UPDATE
    ON OBJECT::[dbo].[titles] TO [guest]
    AS [dbo];

GO
GRANT UPDATE
    ON OBJECT::[dbo].[titleview] TO [guest]
    AS [dbo];

GO
EXECUTE sp_addextendedproperty @name = N'GUID', @value = N'甍ףּ獪䲿¤遻⽪영', @level0type = N'SCHEMA', @level0name = N'dbo', @level1type = N'TABLE', @level1name = N'tbl_Name', @level2type = N'COLUMN', @level2name = N'ColumnName';

GO
