﻿using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net;
using System.Runtime.Serialization;
using System.Text;
using System.Threading;
using System.Xml;
using System.Xml.Linq;
using DacIEServiceCli.IEService;

namespace DacIEServiceCli
{
    public partial class Program
    {
        public void SelectiveExportAction()
        {
            Stopwatch sw = new Stopwatch();

            Console.WriteLine("Export started: {0}", DateTime.Now);

            try
            {
                sw.Start();

                IDACWebService channel = this.CreateWebServiceChannel();
                using (channel as IDisposable)
                {
                    var exportInput = this.GetSelectiveExportInput();
                    Guid resultGuid = channel.SelectiveExport(exportInput);

                    Console.WriteLine("Operation Response: {0}", resultGuid);
                }

            }
            catch (WebException ex)
            {
                if ((ex.Response != null) && (ex.Response is HttpWebResponse))
                {
                    HttpWebResponse response = ex.Response as HttpWebResponse;
                    Console.WriteLine("Service Error: {0} {1} ", response.StatusCode, response.StatusDescription);
                }
                else
                {
                    Console.WriteLine("Error: {0}", ex.Message);
                }
            }
            finally
            {
                sw.Stop();

                Console.WriteLine("Export Submit Complete.  Total time: {0}", sw.Elapsed.ToString());
            }
        }

        private SelectiveExportInput GetSelectiveExportInput()
        {
            SelectiveExportInput ei = new SelectiveExportInput();
            ei.BlobCredentials = Program.GetBlobCredentials(this.blobUrl, this.blobAccessKey, this.blobAccessKeyType);
            ei.ConnectionInfo = this.connectionInfo;
            ei.Tables = GetTableNames().ToArray();
            return ei;
        }

        private IEnumerable<TableName> GetTableNames()
        {
            IList<TableName> tableNames = new List<TableName>();

            using (var stream = new FileStream(this.selectiveExportFileName, FileMode.Open))
            {
                using (XmlReader reader = XmlReader.Create(stream))
                {
                    try
                    {
                        while (reader.ReadToFollowing("TableName"))
                        {
                            reader.ReadToFollowing("SchemaName");
                            string schemaName = reader.ReadElementContentAsString();
                            reader.ReadToFollowing("Name");
                            string name = reader.ReadElementContentAsString();
                            tableNames.Add(new TableName { SchemaName = schemaName, Name = name });
                        }
                    }
                    catch (XmlException ex)
                    {
                        Console.WriteLine("{0}. Please refer to the sample file for the correct format.", ex.Message);
                    }
                }

                return tableNames;
            }
        }
    }
}
